/*********************************************************************************
 * Klasse....: Tabelle 								 *
 *********************************************************************************/
#ifndef TABELLE_H
#define TABELLE_H
#include<stdlib.h>
#include<string.h>

struct Eintrag
{
  char *name;
  void init(char *_name) { name = strdup(_name);}
};

template <class EintragT> class Tabelle
{
  protected:
    int anz;
  public:
    Tabelle() {anz=0;}
    int n() { return anz; }
    virtual EintragT *finde(char *name)=0;
    EintragT *operator[](char *name) { return finde(name); }
    virtual void add(const EintragT& ein)=0;
    // fr Iteratoren
    virtual EintragT *erstes()=0;
    virtual EintragT *naechstes(EintragT *von)=0;

 friend class TabellenIterator<EintragT>;
};

template <class EintragT> class TabellenIterator
{
protected:
   EintragT *akt;
   Tabelle<EintragT> *tabelle;
public:
   TabellenIterator(Tabelle<EintragT> &tab)
   { akt = tab.erstes();
     tabelle = &tab;
   }
   virtual EintragT *operator++()  { return akt=tabelle->naechstes(akt); }
   virtual operator EintragT *()   { return akt; }
   virtual EintragT *lfd()         { return akt; }
};

/* <<<<<<<<<<<<<<<<< Aufloesungstabelle <<<<<<<<<<<<<<<<<<<<<<*/
template <class EintragT,int MAXSIZE> class AufloesungsTabelle : public Tabelle<EintragT>
{
protected:
  EintragT in[MAXSIZE];
public:
  AufloesungsTabelle() {}
  int finde_nr(char *name)
  {
     int i=0;
     int gef;
     while(i<anz && !(gef=(strcmp(name,in[i].name)==0))) i++;
     return (i < anz && gef ? i : -1) ;  
  }
  EintragT *finde(char *name)
  {
     int index = finde_nr(name);
     return ( index >=0 ? &in[index] : NULL );
  } 
  void add(const EintragT& ein)
  {
    if(anz < MAXSIZE)
    {
      in[anz] = ein; anz++;
    }
    else
    {
       cerr << "Aufloesungstabelle::add: Tabelle voll " << endl;
    }
  }
  EintragT *operator[](int i)
  {
    if(i<anz) return &in[i];
    else
    { cerr << "Auflosetabelle[]:Index ausserhalb guetigen Bereichs\n";
      exit(1);
      return NULL;
    }
  }
  EintragT *erstes() { return &in[0]; }
  EintragT *naechstes(EintragT *von)
  { int index = von - in +1;
    return index < anz ? &(in[index]) : NULL;
  }
};


/* <<<<<<<<<<<<<<<<< Tabelle als Liste <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/
template <class EintragT> class TabelleAlsListe: public Tabelle<EintragT>
{
  protected:
   struct Knoten
   {
     EintragT e;
     Knoten *nxt;
     Knoten(const EintragT& ein) { e=ein; nxt=NULL;}
   };
   Knoten *wurzel;
  public:
   TabelleAlsListe() { wurzel = NULL; }
   EintragT *finde(char *name)
   {
     Knoten *akt = wurzel;
     int r;
     while(akt!=NULL && (r=strcmp(name,akt->e.name))>0) akt = akt->nxt;
     if(akt!=NULL && r == 0) return &(akt->e);
     else  return NULL;
   }
   void add(const EintragT& ein)
   {
      Knoten **akt = &wurzel;
      Knoten *h;
      int r;
      while(*akt!=NULL && (r=strcmp(ein.name,(*akt)->e.name))>0) akt = &((*akt)->nxt);
      h = new Knoten(ein);
      h->nxt = *akt;
      *akt = h;
      anz++;
   }
   EintragT *erstes() { return &(wurzel->e); }
   EintragT *naechstes(EintragT *akt) { return akt!=NULL ? &(((Knoten *)akt)->nxt->e) : NULL; }
};

#endif


