/**************************************************************************
 * Headerfille....: zeit.h                                                *
 * Zweck..........: Datum- und Zeitrechnung                               *
 * Author.........: Rudolf Weber                                          *
 **************************************************************************/
#ifndef ZEIT_H
#define ZEIT_H
#include<sys/time.h>
inline int schaltjahr(int jahr) { return ((jahr) % 4 == 0 && ((jahr) % 100 != 0 || (jahr) % 400 == 0)); }

const unsigned short int __mon_lengths[2][12] =
  {
    /* Normal years.  */
    { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
    /* Leap years.  */
    { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
  };

inline short monatlang(int jahr,int monat)
{ return __mon_lengths[schaltjahr(jahr)][monat-1];
}

extern char *wochentag[];
const sekjetag = 24*60*60;

class Datum
{
public:
  short tag;
  short monat;
  int jahr;

  Datum() { tag = 1; monat = 1; jahr = 1970; }
  Datum(int t,int m,int j) { tag=t;monat=m;jahr = j; }
  const char *parse(const char *str,const char **pos);
  void print(ostream &ostr) 
    {
      ostr << tag << '.' << monat << '.' << jahr;
    }
  // Morgenoperator
  Datum &operator++();
  // Abstand der Kalendertage 
  int operator-(const Datum& von) const;
  // Zaehlt ab Tage im Kalender weiter
  Datum operator+(int ab);
  int cmp(const Datum& d2) const
      {
	  if(jahr<d2.jahr) return -1;
          else if(jahr>d2.jahr) return 1;
	  else if(monat<d2.monat) return -1;
          else if(monat>d2.monat) return 1;
	  else if(tag<d2.tag) return -1;
          else if(tag>d2.tag) return 1;
	  else return 0;
      }
};

inline time_t sekzeit(const Datum& d) // Sekunden seit 1.1.1970
{
  if(d.jahr < 1970) 
  {
      cerr << "sekzeit Zeiten frueher als 1970 sind verboten!";
      return -1;
  } 
  return (d-Datum(1,1,1970))*sekjetag;
}

inline time_t sekzeit696(const Datum& d) // Sekunden seit 1.6.1996
{
  if(d.jahr <= 1996 && (d.jahr!=1996 || d.monat < 6)) 
  {
      cerr << "sekzeit Zeiten frueher als 1996 sind verboten!" << endl;
      return -1;
  } 
  return (d-Datum(1,6,1996)+9648)*sekjetag;
}

inline ostream& operator<<(ostream& str,Datum &d)
{
  d.print(str);
  return str;
}

int heutig(Datum &h); // liefert Wochentag
#endif








